#include <QMC5883L.h>

#include <NewPing.h>
#include <DHT.h>
#include <Wire.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_BME280.h>
#include <AFMotor.h>

#define DHTPIN 13
#define DHTTYPE DHT11
#define TRIGGER_PIN 10
#define ECHO_PIN 2
#define SEALEVELPRESSURE_HPA (1013.25)

DHT dht(DHTPIN, DHTTYPE);
NewPing sonar(TRIGGER_PIN, ECHO_PIN);
Adafruit_BME280 bme;
QMC5883L compass;
AF_DCMotor motor1(4);
AF_DCMotor motor2(1);
AF_DCMotor motor3(3);
AF_DCMotor motor4(2);



int pinPhotoresistor = A0;
int pinGassensor = A1;

int forwardSpeed = 255;
int distance = 0;
char command = 's';
bool wayBlocked = true;

int intervalBluetooth = 1500;
long oldTimeBluetooth = 0;
long oldTimeUltrasonic = 0;


void setup() {

  Serial.begin(9600);
  Wire.begin();
  dht.begin(); //initalise DHT11 Temperature and Humidity
  bme.begin(0x76); //initialise I2C Interface of BME280 at given address
  compass.init(); //initialise GY-271
}

void loop() {

  //read serial interface
  if (Serial.available() > 0)
    command = Serial.read();

  //checks distance
  if (millis() - oldTimeUltrasonic  >= 50) {
    distance = sonar.ping_cm();
    oldTimeUltrasonic = millis();
  }

  //slows down/stops motors when close to an obstacle
  //NO_ECHO = 0: if an obstacle is more than 5m away, sensor returns 0
  if (distance < 15 && distance > NO_ECHO)
    forwardSpeed = 15 * distance;
  else
    forwardSpeed = 255;
  if (distance <= 5 && distance > NO_ECHO)
    wayBlocked = true;
  else
    wayBlocked = false;

  //motor control
  if (command == 'f' && wayBlocked == true)
    brake();
  else if (command == 'f' && wayBlocked == false)
    forward();
  else if (command == 'b')
    back();
  else if (command == 'l')
    left();
  else if (command == 'r')
    right();
  else if (command == 's')
    brake();

  //read all sensors and print it to the serial interface
  //with the current setup (without DHT11 the block takes about 14ms, with DHT11 it takes up to 20ms longer
  if (millis() - oldTimeBluetooth >= intervalBluetooth) {
    oldTimeBluetooth = millis();
    //Serial.println(millis());

    double lightIntensity = calculateLightIntensity(analogRead(pinPhotoresistor));
    //float temperature = dht.readTemperature(); //in °C
    //float humidity = dht.readHumidity(); //in %
    float temperature = bme.readTemperature(); //in °C
    float humidity = bme.readHumidity(); //in %
    double pressure = bme.readPressure() / 100.0F; //in hPa
    int gasStrength = analogRead(pinGassensor);

    //read values in uG
    int xG, yG, zG, t;
    if (compass.ready()) {
      int heading = compass.readHeading();
      int r = compass.readRaw(&xG, &yG, &zG, &t);
    }
    double xT = xG / 100; double yT = yG / 100; double zT = zG / 100; //convert from uG to uT
    double absoluteMagneticFieldStrength = sqrt((xT * xT) + (yT * yT) + (zT * zT));

    Serial.print(lightIntensity);
    Serial.print("|");
    Serial.print(temperature);
    Serial.print("|");
    Serial.print(humidity);
    Serial.print("|");
    Serial.print(pressure);
    Serial.print("|");
    Serial.print(absoluteMagneticFieldStrength);
    Serial.print("|");
    Serial.print(gasStrength);
    Serial.print("|");
    Serial.println(distance);
    //Serial.println(millis());
  }
}

void forward()
{
  motor1.setSpeed(forwardSpeed);
  motor1.run(FORWARD);
  motor2.setSpeed(forwardSpeed);
  motor2.run(FORWARD);
  motor3.setSpeed(forwardSpeed);
  motor3.run(FORWARD);
  motor4.setSpeed(forwardSpeed);
  motor4.run(FORWARD);
}

void back()
{
  motor1.setSpeed(255);
  motor1.run(BACKWARD);
  motor2.setSpeed(255);
  motor2.run(BACKWARD);
  motor3.setSpeed(255);
  motor3.run(BACKWARD);
  motor4.setSpeed(255);
  motor4.run(BACKWARD);
}

void left()
{
  motor1.setSpeed(255);
  motor1.run(BACKWARD);
  motor2.setSpeed(255);
  motor2.run(BACKWARD);
  motor3.setSpeed(255);
  motor3.run(FORWARD);
  motor4.setSpeed(255);
  motor4.run(FORWARD);
}

void right()
{
  motor1.setSpeed(255);
  motor1.run(FORWARD);
  motor2.setSpeed(255);
  motor2.run(FORWARD);
  motor3.setSpeed(255);
  motor3.run(BACKWARD);
  motor4.setSpeed(255);
  motor4.run(BACKWARD);
}

void brake()
{
  motor1.setSpeed(0);
  motor1.run(RELEASE);
  motor2.setSpeed(0);
  motor2.run(RELEASE);
  motor3.setSpeed(0);
  motor3.run(RELEASE);
  motor4.setSpeed(0);
  motor4.run(RELEASE);
}

/*
   https://medium.com/@jxxcarlson/measuring-light-intensity-with-arduino-a575765c0862
*/
double calculateLightIntensity (int rawADCPinPhotoresistor) {
  const double k = 5.0 / 1024;
  const double luxFactor = 500000;
  const double R2 = 10000;
  const double B = 1.3 * pow(10.0, 7);
  const double m = -1.4;

  double V2 = k * rawADCPinPhotoresistor;
  double R1 = (5.0 / V2 - 1) * R2;
  double lux = B * pow(R1, m);
  return lux;
}
